/*
  Akana R1 GateLAB v1

  Logic Gate Training Code for METE HOCA Akana R1
  
  Board : METE HOCA Akana R1
  Dock  : -
  Core  : earlephilhower / arduino-pico (https://github.com/earlephilhower/arduino-pico)
  Gerekli kütüphaneler: U8G2 (https://github.com/olikraus/u8g2)
  Settings: Select METE HOCA Akana R1 as the board.
	    Set the I2C selector to the I2C0 position.

  Source: METE HOCA
  
  METE HOCA, 19 May 2024
  IG : https://www.instagram.com/metehoca/
  YT : https://www.youtube.com/MeteHoca
  WEB: https://www.metehoca.com/
*/

#include "Akana.h"
#include "Gates.h"

const char* title = "Akana R1 GateLAB";

int gate = 0;

void setup() {
  u8g2.begin();
  u8g2.setFont(u8g2_font_6x12_me);
  u8g2.setFontMode(1);
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setContrast(255);
  
  pinMode(BTN_UP, INPUT);
  pinMode(BTN_DOWN, INPUT);
  pinMode(BTN_LEFT, INPUT);
  pinMode(BTN_RIGHT, INPUT);
  pinMode(BTN_BACK, INPUT);
  pinMode(BTN_ENTER, INPUT);

  attachInterrupt(digitalPinToInterrupt(BTN_LEFT), gatePREV, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_RIGHT), gateNEXT, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_ENTER), gateNEXT, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_BACK), gatePREV, RISING);

  logoAkana();
}

void loop() {
  if (gate == 0) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameAND))) / 2), 56, nameAND);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateAND);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) && digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.sendBuffer();
  }
  if (gate == 1) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameOR))) / 2), 56, nameOR);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateOR);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.sendBuffer();
  }
  if (gate == 2) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameNAND))) / 2), 56, nameNAND);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateNAND);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) && digitalRead(BTN_DOWN)) u8g2.print("0"); else u8g2.print("1");
    u8g2.sendBuffer();
  }
  if (gate == 3) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameNOR))) / 2), 56, nameNOR);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateNOR);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("0"); else u8g2.print("1");
    u8g2.sendBuffer();
  }
  if (gate == 4) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameXOR))) / 2), 56, nameXOR);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateXOR);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) != digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.sendBuffer();
  }
  if (gate == 5) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameXNOR))) / 2), 56, nameXNOR);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateXNOR);
    showArrows();
    u8g2.setCursor(10,18);
    if (digitalRead(BTN_UP)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(10,38);
    if (digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) != digitalRead(BTN_DOWN)) u8g2.print("0"); else u8g2.print("1");
    u8g2.sendBuffer();
  }
  if (gate == 6) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameNOT))) / 2), 56, nameNOT);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateNOT);
    showArrows2();
    u8g2.setCursor(10,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("0"); else u8g2.print("1");
    u8g2.sendBuffer();
  }
  if (gate == 7) {
    u8g2.clearBuffer();
    showTitle();
    u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(nameBUFFER))) / 2), 56, nameBUFFER);
    u8g2.setFont(u8g2_font_t0_18_me);
    u8g2.drawXBMP(19, 12, 90, 44, gateBUFFER);
    showArrows2();
    u8g2.setCursor(10,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.setCursor(110,28);
    if (digitalRead(BTN_UP) || digitalRead(BTN_DOWN)) u8g2.print("1"); else u8g2.print("0");
    u8g2.sendBuffer();
  }
  delay(50);
}

void showTitle() {
  u8g2.setFont(u8g2_font_6x12_me);
  u8g2.drawRBox(0,0,127,12,5);
  u8g2.setDrawColor(0);
  u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(title))) / 2), 2, title);
  u8g2.setDrawColor(1);
  u8g2.drawTriangle(11,59, 18,55, 18,63);
  u8g2.drawTriangle(116,59, 109,55, 109,63);
}

void showArrows() {
  u8g2.drawTriangle(4,20, 0,27, 8,27);
  u8g2.drawTriangle(1,41, 8,41, 4,48);
}

void showArrows2() {
  u8g2.drawTriangle(4,26, 0,33, 8,33);
  u8g2.drawTriangle(1,36, 8,36, 4,43);
}

void gatePREV() {
  gate--;
  if (gate == -1) gate = 7;
}

void gateNEXT() {
  gate++;
  if (gate == 8) gate = 0;
}