/*
  Akana OS v0.1 (BETA)

  Simple Firmware for METE HOCA Akana R1
  
  Board : METE HOCA Akana R1
  Dock  : -
  Core  : earlephilhower / arduino-pico (https://github.com/earlephilhower/arduino-pico)
  Required Libraries: U8G2 (https://github.com/olikraus/u8g2)
                      TaskManagerIO (https://github.com/davetcc/TaskManagerIO)
  Settings: Select METE HOCA Akana R1 as the board.
            Set the I2C selector to the I2C0 position.

Source: METE HOCA
  
  METE HOCA, 19 Mayıs 2024
  IG : https://www.instagram.com/metehoca/
  YT : https://www.youtube.com/MeteHoca
  WEB: https://www.metehoca.com/
*/

#include "Akana.h"

bool menuLoop = false;
byte menuPosition[] = {10, 15, 27, 39, 51};

const char* mainMenu_title = "AKANA OS v0.1";
const char* pinControl_title = "CONTROL A PIN";
const char* pinPWM_title = "PWM A PIN";
const char* pinBlink_title = "BLINK A PIN";
const char* pinFade_title = "FADE A PIN";
const char* pinServo_title = "SERVO CONTROL";

const char* mainMenu[] = {pinControl_title, pinPWM_title, pinBlink_title, pinFade_title, pinServo_title, "REBOOT", "BOOTSEL"};   // 0
const char* pinControl[] = {"OUTPUT > HIGH", "OUTPUT > LOW", "OUTPUT", "INPUT", "INPUT_PULLUP", "INPUT_PULLDOWN"}; // 1
const char* pinPWM[] = {"255", "192", "128", "64", "32", "16", "8", "0"};                                          // 2
const char* pinBlink[] = {"1000ms", "500ms", "100ms", "50ms", "25ms", "OFF"};                                      // 3
const char* pinFade[] = {"FAST", "SLOW", "OFF"};                                                                   // 4
const char* pinServo[] = {"0", "30", "45", "60", "90", "120", "135", "150", "180"};                                // 5

const byte pinNumberNormal[30] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29};
const char* pinNamesNormal[30] = {"GP0  (UART0 TX)", "GP1  (UART0 RX)", "GP2", "GP3", "GP4  (I2C0 SDA)", "GP5  (I2C0 SCL)", "GP6  (I2C1 SDA)", "GP7  (I2C1 SCL)", "GP8  (UART1 TX)", "GP9  (UART1 RX)", "GP10 (SPI1 SCK)", "GP11 (SPI1 TX)", "GP12 (SPI1 RX)", "GP13 (SPI1 CSn)", "GP14 (BTN_ENTER)", "GP15 (BTN_BACK)", "GP16 (SPI0 RX)", "GP17 (SPI0 CSn)", "GP18 (SPI0 SCK)", "GP19 (SPI0 TX)", "GP20 (BTN_LEFT)", "GP21 (BTN_RIGHT)", "GP22 (BTN_UP)", "GP23 (BTN_DOWN)", "GP24", "GP25 (LED)", "GP26 (A0)", "GP27 (A1)", "GP28 (A2)", "GP29 (A3)"};
const byte pinNumberSorted[30] = {25, 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 16, 17, 18, 19, 24, 26, 27, 28, 29, 14, 15, 20, 21, 22, 23, 4, 5, 6, 7};
const char* pinNamesSorted[30] = {"GP25 (LED)", "GP0  (UART0 TX)", "GP1  (UART0 RX)", "GP2", "GP3", "GP8  (UART1 TX)", "GP9  (UART1 RX)", "GP10 (SPI1 SCK)", "GP11 (SPI1 TX)", "GP12 (SPI1 RX)", "GP13 (SPI1 CSn)", "GP16 (SPI0 RX)", "GP17 (SPI0 CSn)", "GP18 (SPI0 SCK)", "GP19 (SPI0 TX)", "GP24", "GP26 (A0)", "GP27 (A1)", "GP28 (A2)", "GP29 (A3)", "GP14 (BTN_ENTER)", "GP15 (BTN_BACK)", "GP20 (BTN_LEFT)", "GP21 (BTN_RIGHT)", "GP22 (BTN_UP)", "GP23 (BTN_DOWN)", "GP4  (I2C0 SDA)", "GP5  (I2C0 SCL)", "GP6  (I2C1 SDA)", "GP7  (I2C1 SCL)"};
const byte pinNumberShort[20] = {25, 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 16, 17, 18, 19, 24, 26, 27, 28, 29};
const char* pinNamesShort[20] = {"GP25 (LED)", "GP0  (UART0 TX)", "GP1  (UART0 RX)", "GP2", "GP3", "GP8  (UART1 TX)", "GP9  (UART1 RX)", "GP10 (SPI1 SCK)", "GP11 (SPI1 TX)", "GP12 (SPI1 RX)", "GP13 (SPI1 CSn)", "GP16 (SPI0 RX)", "GP17 (SPI0 CSn)", "GP18 (SPI0 SCK)", "GP19 (SPI0 TX)", "GP24", "GP26 (A0)", "GP27 (A1)", "GP28 (A2)", "GP29 (A3)"};

// 0=NOT SELECTED, 1=INPUT, 2=INPUT_PULLUP, 3=INPUT_PULLDOWN, 4=OUTPUT_12MA
byte pinStateSorted[30] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0};
byte pinStateNormal[30] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0};
byte pinStateShort[20] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

byte pinPWMStateShort[20]   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  // 0=OFF, 1=255, 2=192, 3=128, 4=64, 5=32, 6=16, 7=8
byte pinBlinkStateShort[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  // 0=OFF, 1=1000ms, 2=500ms, 3=100ms, 4=50ms, 5=25ms
byte pinFadeStateShort[20]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  // 0=OFF, 1=5ms, 2=10ms
byte pinServoStateShort[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  // 0-180

byte mainMenu_length, pinControl_length, pinPWM_length, pinBlink_length, pinFade_length, pinServo_length;
int fade5, fade10;
volatile byte menu = 0;
byte selectedPin = 0;
byte activeMenu = 0;
byte activeMenuLength = 0;

int taskTimer_1000, taskTimer_500, taskTimer_100, taskTimer_50, taskTimer_25, taskTimer_10, taskTimer_5;

void setup() {
  mainMenu_length = sizeof(mainMenu) / sizeof(mainMenu[0]); // Menü boyutunu hesaplıyoruz.
  pinControl_length = sizeof(pinControl) / sizeof(pinControl[0]);
  pinPWM_length = sizeof(pinPWM) / sizeof(pinPWM[0]);
  pinBlink_length = sizeof(pinBlink) / sizeof(pinBlink[0]);
  pinFade_length = sizeof(pinFade) / sizeof(pinFade[0]);
  pinServo_length = sizeof(pinServo) / sizeof(pinServo[0]);

  u8g2.begin();
  u8g2.setFont(u8g2_font_6x12_me);
  u8g2.setFontMode(1);
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setContrast(255);
  
  pinMode(BTN_UP, INPUT);
  pinMode(BTN_DOWN, INPUT);
  pinMode(BTN_LEFT, INPUT);
  pinMode(BTN_RIGHT, INPUT);
  pinMode(BTN_BACK, INPUT);
  pinMode(BTN_ENTER, INPUT);

  logoAkana();

  attachInterrupt(digitalPinToInterrupt(BTN_UP), MenuUP, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_DOWN), MenuDN, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_LEFT), MenuBC, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_RIGHT), MenuEN, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_ENTER), MenuEN, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_BACK), MenuBC, RISING);

  showMenu();
  taskTimer();

  myservo.attach(2);
}

void loop() {
  taskManager.runLoop();
}

void MenuUP() {
  if (menu > 0) menu--; else if (menuLoop) menu = activeMenuLength-1;
  showMenu();
}

void MenuDN() {
  if (menu != activeMenuLength-1) menu++; else if (menuLoop) menu = 0;
  showMenu();
}

void MenuEN() {
  if (activeMenu == 10) pinServoMenu();
  if (activeMenu == 9) {activeMenu = 10; selectedPin = menu; menu = 0;}
  if (activeMenu == 8) pinFadeMenu();
  if (activeMenu == 7) {activeMenu = 8; selectedPin = menu; menu = 0;}
  if (activeMenu == 6) pinBlinkMenu();
  if (activeMenu == 5) {activeMenu = 6; selectedPin = menu; menu = 0;}
  if (activeMenu == 4) pinPWMMenu();
  if (activeMenu == 3) {activeMenu = 4; selectedPin = menu; menu = 0;}
  if (activeMenu == 2) pinControlMenu();
  if (activeMenu == 1) {activeMenu = 2; selectedPin = menu; menu = 0;}
  if (activeMenu == 0) theMainMenu();
  showMenu();
}

void MenuBC() {
  if (activeMenu == 1) {activeMenu = 0; menu = 0;}
  if (activeMenu == 2) {activeMenu = 1; menu = 0;}
  if (activeMenu == 3) {activeMenu = 0; menu = 0;}
  if (activeMenu == 4) {activeMenu = 3; menu = 0;}
  if (activeMenu == 5) {activeMenu = 0; menu = 0;}
  if (activeMenu == 6) {activeMenu = 5; menu = 0;}
  if (activeMenu == 7) {activeMenu = 0; menu = 0;}
  if (activeMenu == 8) {activeMenu = 7; menu = 0;}
  if (activeMenu == 9) {activeMenu = 0; menu = 0;}
  if (activeMenu == 10) {activeMenu = 9; menu = 0;}
  showMenu();
}

void showMenu() {
  if (activeMenu == 0) {activeMenuLength = mainMenu_length; theMenu(mainMenu_title, mainMenu, menu);}
  if (activeMenu == 1) {activeMenuLength = 20; theMenu(pinControl_title, pinNamesShort, menu);}
  if (activeMenu == 2) {activeMenuLength = pinControl_length; theMenu(pinNamesShort[selectedPin], pinControl, menu);}
  if (activeMenu == 3) {activeMenuLength = 20; theMenu(pinPWM_title, pinNamesShort, menu);}
  if (activeMenu == 4) {activeMenuLength = pinPWM_length; theMenu(pinNamesShort[selectedPin], pinPWM, menu);}
  if (activeMenu == 5) {activeMenuLength = 20; theMenu(pinBlink_title, pinNamesShort, menu);}
  if (activeMenu == 6) {activeMenuLength = pinBlink_length; theMenu(pinNamesShort[selectedPin], pinBlink, menu);}
  if (activeMenu == 7) {activeMenuLength = 20; theMenu(pinFade_title, pinNamesShort, menu);}
  if (activeMenu == 8) {activeMenuLength = pinFade_length; theMenu(pinNamesShort[selectedPin], pinFade, menu);}
  if (activeMenu == 9) {activeMenuLength = 20; theMenu(pinServo_title, pinNamesShort, menu);}
  if (activeMenu == 10) {activeMenuLength = pinServo_length; theMenu(pinNamesShort[selectedPin], pinServo, menu);}
}

void theMenu(const char* menuTitle, const char* menuName[], byte menuSelected) {
  u8g2.clearBuffer();
  u8g2.drawRFrame(0,0,127,63,5);
  u8g2.drawRBox(0,0,127,12,5);
  u8g2.drawBox(0,8,127,4);
  
  u8g2.setDrawColor(0);
  u8g2.drawUTF8(((128 - (u8g2.getUTF8Width(menuTitle))) / 2), 2, menuTitle);
  u8g2.setDrawColor(1);

  if (menuSelected < 4) {
    for (byte x = 0; x < 4; x++) {
      if (x == menuSelected) u8g2.drawRBox(2, menuPosition[menuSelected+1]-2, 123, 12, 3);
      if (x == menuSelected) u8g2.setDrawColor(0);
      if (x < activeMenuLength) u8g2.drawUTF8(menuPosition[0], menuPosition[x+1], menuName[x]);
      if (x == menuSelected) u8g2.setDrawColor(1);
    }
  } else {
    for (byte x = 4; x > 0; x--) {
      if (x == 4) u8g2.drawRBox(2, menuPosition[4]-2, 123, 12, 3);
      if (x == 4) u8g2.setDrawColor(0);
      u8g2.drawUTF8(menuPosition[0], menuPosition[x], menuName[menuSelected - 4 + x]);
      if (x == 4) u8g2.setDrawColor(1);
    }
  }
  u8g2.sendBuffer();
}

void theMainMenu() {
  if (menu == 0) {activeMenu = 1; menu = 0;} // pinControl
  if (menu == 1) {activeMenu = 3; menu = 0;} // pinPWM
  if (menu == 2) {activeMenu = 5; menu = 0;} // pinBlink
  if (menu == 3) {activeMenu = 7; menu = 0;} // pinFade
  if (menu == 4) {activeMenu = 9; menu = 0;} // pinServo
  if (menu == 5) rp2040.reboot();            // RESET
  if (menu == 6) {                           // BOOTSEL
    u8g2.clearBuffer();
    u8g2.drawXBMP(0, 14, 128, 36, AKANA);
    u8g2.sendBuffer();
    rp2040.rebootToBootloader();
  }
}

void pinControlMenu() {
  if (menu == 0) {
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    digitalWriteFast(pinNumberShort[selectedPin], HIGH);
  }
  if (menu == 1) {
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    digitalWriteFast(pinNumberShort[selectedPin], LOW);
  }
  if (menu == 2) {
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
  }
  if (menu == 3) {
    pinMode(pinNumberShort[selectedPin], INPUT);
    pinStateShort[selectedPin] = 1;
  }
  if (menu == 4) {
    pinMode(pinNumberShort[selectedPin], INPUT_PULLUP);
    pinStateShort[selectedPin] = 2;
  }
  if (menu == 5) {
    pinMode(pinNumberShort[selectedPin], INPUT_PULLDOWN);
    pinStateShort[selectedPin] = 3;
  }
}

void pinPWMMenu() {
  if (menu == 0) { //255
    pinPWMStateShort[selectedPin] = 1;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 255);
  }
  if (menu == 1) { //192
    pinPWMStateShort[selectedPin] = 2;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 192);
  }
  if (menu == 2) { //128
    pinPWMStateShort[selectedPin] = 3;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 128);
  }
  if (menu == 3) { //64
    pinPWMStateShort[selectedPin] = 4;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 64);
  }
  if (menu == 4) { //32
    pinPWMStateShort[selectedPin] = 5;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 32);
  }
  if (menu == 5) { //16
    pinPWMStateShort[selectedPin] = 6;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 16);
  }
  if (menu == 6) { //8
    pinPWMStateShort[selectedPin] = 7;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 8);
  }
  if (menu == 7) { //0
    pinPWMStateShort[selectedPin] = 0;
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    analogWrite(pinNumberShort[selectedPin], 0);
  }
}

void pinBlinkMenu() {
  if (menu == 0) { // 1000ms
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    pinBlinkStateShort[selectedPin] = 1;
  }
  if (menu == 1) { // 500ms
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    pinBlinkStateShort[selectedPin] = 2;
  }
  if (menu == 2) { // 100ms
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    pinBlinkStateShort[selectedPin] = 3;
  }
  if (menu == 3) { // 50ms
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    pinBlinkStateShort[selectedPin] = 4;
  }
  if (menu == 4) { // 25ms
    pinMode(pinNumberShort[selectedPin], OUTPUT_12MA);
    pinStateShort[selectedPin] = 4;
    pinBlinkStateShort[selectedPin] = 5;
  }
  if (menu == 5) { // OFF
    pinBlinkStateShort[selectedPin] = 0;
    digitalWriteFast(pinNumberShort[selectedPin], LOW);
  }
}

void pinFadeMenu() {
  if (menu == 0) { // FAST
    pinFadeStateShort[selectedPin] = 1;
  }
  if (menu == 1) { // SLOW
    pinFadeStateShort[selectedPin] = 2;
  }
  if (menu == 2) { // OFF
    pinFadeStateShort[selectedPin] = 0;
    analogWrite(pinNumberShort[selectedPin], 0);
  }
}

void pinServoMenu() {
  if (menu == 0) {pinServoStateShort[selectedPin] = 0; myservo.write(pinServoStateShort[selectedPin]);}   // 0
  if (menu == 1) {pinServoStateShort[selectedPin] = 30; myservo.write(pinServoStateShort[selectedPin]);}  // 30
  if (menu == 2) {pinServoStateShort[selectedPin] = 45; myservo.write(pinServoStateShort[selectedPin]);}  // 45
  if (menu == 3) {pinServoStateShort[selectedPin] = 60; myservo.write(pinServoStateShort[selectedPin]);}  // 60
  if (menu == 4) {pinServoStateShort[selectedPin] = 90; myservo.write(pinServoStateShort[selectedPin]);}  // 90
  if (menu == 5) {pinServoStateShort[selectedPin] = 120; myservo.write(pinServoStateShort[selectedPin]);} // 120
  if (menu == 6) {pinServoStateShort[selectedPin] = 135; myservo.write(pinServoStateShort[selectedPin]);} // 135
  if (menu == 7) {pinServoStateShort[selectedPin] = 150; myservo.write(pinServoStateShort[selectedPin]);} // 150
  if (menu == 8) {pinServoStateShort[selectedPin] = 180; myservo.write(pinServoStateShort[selectedPin]);} // 180
}

void taskTimer() {
  taskTimer_1000 = taskManager.schedule(repeatMillis(1000), [] { // 1000ms
    for (int y = 0; y < 20; y++) if (pinBlinkStateShort[y] == 1) digitalToggle(pinNumberShort[y]); // pinBlink
  });
  taskTimer_500 = taskManager.schedule(repeatMillis(500), [] {   // 500ms
    for (int y = 0; y < 20; y++) if (pinBlinkStateShort[y] == 2) digitalToggle(pinNumberShort[y]); // pinBlink
  });
  taskTimer_100 = taskManager.schedule(repeatMillis(100), [] {   // 100ms
    for (int y = 0; y < 20; y++) if (pinBlinkStateShort[y] == 3) digitalToggle(pinNumberShort[y]); // pinBlink
  });
  taskTimer_50 = taskManager.schedule(repeatMillis(50), [] {     // 50ms
    for (int y = 0; y < 20; y++) if (pinBlinkStateShort[y] == 4) digitalToggle(pinNumberShort[y]); // pinBlink
  });
  taskTimer_25 = taskManager.schedule(repeatMillis(25), [] {     // 25ms
    for (int y = 0; y < 20; y++) if (pinBlinkStateShort[y] == 5) digitalToggle(pinNumberShort[y]); // pinBlink
  });
  taskTimer_10 = taskManager.schedule(repeatMillis(10), [] {     // 10ms
    fade10++; if (fade10==511) fade10 = 0; // pinFade
    for (int fd10 = 0; fd10 < 20; fd10++) {
      if (pinFadeStateShort[fd10] == 2) {
        if (fade10 < 256) analogWrite(pinNumberShort[fd10], fade10);
        if (fade10 > 255) analogWrite(pinNumberShort[fd10], 510-fade10);
      }
    }
  });
  taskTimer_5 = taskManager.schedule(repeatMillis(5), [] {       // 5ms
    fade5++; if (fade5==511) fade5 = 0; // pinFade
    for (int fd5 = 0; fd5 < 20; fd5++) {
      if (pinFadeStateShort[fd5] == 1) {
        if (fade5 < 256) analogWrite(pinNumberShort[fd5], fade5);
        if (fade5 > 255) analogWrite(pinNumberShort[fd5], 510-fade5);
      }
    }
  });
}