/*
  Akana R1 Nano Bird v1

  Flappy Bird Game for METE HOCA Akana R1

  Board : METE HOCA Akana R1
  Dock  : -
  Core  : earlephilhower / arduino-pico (https://github.com/earlephilhower/arduino-pico)
  Required Libraries: Adafruit SSD1306 (https://github.com/adafruit/Adafruit_SSD1306)
                      Adafruit GFX (https://github.com/adafruit/Adafruit-GFX-Library)
  Settings: Select METE HOCA Akana R1 as the board.
            Set the I2C selector to the I2C0 position.

  Source: u/richardathome / METE HOCA
  
  METE HOCA, 19 May 2024
  IG : https://www.instagram.com/metehoca/
  YT : https://www.youtube.com/MeteHoca
  WEB: https://www.metehoca.com/
*/

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

Adafruit_SSD1306 display(128, 64, &Wire, -1);

#define FLAP_BUTTON  14
#define SPRITE_HEIGHT   16
#define SPRITE_WIDTH    16
#define GAME_SPEED 50

static const unsigned char PROGMEM wing_down_bmp[] =
{ 0b00000000, 0b00000000,
  0b00000000, 0b00000000,
  0b00000011, 0b11000000,
  0b00011111, 0b11110000,
  0b00111111, 0b00111000,
  0b01111111, 0b11111110,
  0b11111111, 0b11000001,
  0b11011111, 0b01111110,
  0b11011111, 0b01111000,
  0b11011111, 0b01111000,
  0b11001110, 0b01111000,
  0b11110001, 0b11110000,
  0b01111111, 0b11100000,
  0b00111111, 0b11000000,
  0b00000111, 0b00000000,
  0b00000000, 0b00000000,
};

static const unsigned char PROGMEM wing_up_bmp[] =
{ 0b00000000, 0b00000000,
  0b00000000, 0b00000000,
  0b00000011, 0b11000000,
  0b00011111, 0b11110000,
  0b00111111, 0b00111000,
  0b01110001, 0b11111110,
  0b11101110, 0b11000001,
  0b11011111, 0b01111110,
  0b11011111, 0b01111000,
  0b11111111, 0b11111000,
  0b11111111, 0b11111000,
  0b11111111, 0b11110000,
  0b01111111, 0b11100000,
  0b00111111, 0b11000000,
  0b00000111, 0b00000000,
  0b00000000, 0b00000000,
};

int game_state = 1; // 0 = game over screen, 1 = in game
int score = 0; // current game score
int high_score = 0; // highest score since the nano was reset
int bird_x = (int)display.width() / 4; // birds x position (along) - initialised to 1/4 the way along the screen
int bird_y; // birds y position (down)
int momentum = 0; // how much force is pulling the bird down
int wall_x[2]; // an array to hold the walls x positions
int wall_y[2]; // an array to hold the walls y positions
int wall_gap = 30; // size of the wall wall_gap in pixels
int wall_width = 10; // width of the wall in pixels

void setup() {
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.setTextColor(WHITE);
  display.clearDisplay();
  pinMode(FLAP_BUTTON, INPUT);
  randomSeed(analogRead(0));
}

void loop() {
  if (game_state == 0) {
    display.clearDisplay();
    if (digitalRead(FLAP_BUTTON) == HIGH) momentum = -4;
    momentum += 1;
    bird_y += momentum;

    if (bird_y < 0 ) bird_y = 0;

    if (bird_y > display.height() - SPRITE_HEIGHT) {
      bird_y = display.height() - SPRITE_HEIGHT;
      momentum = -2;
    }

    if (momentum < 0) {
      if (random(2) == 0) {
        display.drawBitmap(bird_x, bird_y, wing_down_bmp, 16, 16, WHITE);
      }
      else {
        display.drawBitmap(bird_x, bird_y, wing_up_bmp, 16, 16, WHITE);
      }
    }
    else {
      display.drawBitmap(bird_x, bird_y, wing_up_bmp, 16, 16, WHITE);
    }

    for (int i = 0 ; i < 2; i++) {
      display.fillRect(wall_x[i], 0, wall_width, wall_y[i], WHITE);
      display.fillRect(wall_x[i], wall_y[i] + wall_gap, wall_width, display.height() - wall_y[i] + wall_gap, WHITE);
      if (wall_x[i] < 0) {
        wall_y[i] = random(0, display.height() - wall_gap);
        wall_x[i] = display.width();
      }
      if (wall_x[i] == bird_x) {
        score++;
        high_score = max(score, high_score);
      }

      if (
        (bird_x + SPRITE_WIDTH > wall_x[i] && bird_x < wall_x[i] + wall_width)
        &&
        (bird_y < wall_y[i] || bird_y + SPRITE_HEIGHT > wall_y[i] + wall_gap)
      ) {
        
        display.display();
        delay(500);
        game_state = 1; 
      }
      wall_x[i] -= 4;
    }
    boldTextAtCenter(0, (String)score);
    display.display();
    delay(GAME_SPEED);
  }
  else {
    screenWipe(10);
    outlineTextAtCenter(1, "AKANA R1 NANO BIRD");
    textAtCenter(display.height() / 2 - 8, "GAME OVER");
    textAtCenter(display.height() / 2, String(score));
    boldTextAtCenter(display.height() - 16, "HIGH SCORE");
    boldTextAtCenter(display.height()  - 8, String(high_score));
    display.display();
    while (digitalRead(FLAP_BUTTON) == LOW);

    bird_y = display.height() / 2;
    momentum = -4;
    wall_x[0] = display.width() ;
    wall_y[0] = display.height() / 2 - wall_gap / 2;
    wall_x[1] = display.width() + display.width() / 2;
    wall_y[1] = display.height() / 2 - wall_gap / 1;
    score = 0;
    while (digitalRead(FLAP_BUTTON) == HIGH);
    screenWipe(10);
    game_state = 0;
  }
}

void screenWipe(int speed) {
  for (int i = 0; i < display.height(); i += speed) {
    display.fillRect(0, i, display.width(), speed, WHITE);
    display.display();
  }
  for (int i = 0; i < display.height(); i += speed) {
    display.fillRect(0, i, display.width(), speed, BLACK);
    display.display();
  }
}

void textAt(int x, int y, String txt) {
  display.setCursor(x, y);
  display.print(txt);
}

void textAtCenter(int y, String txt) {
  textAt(display.width() / 2 - txt.length() * 3, y, txt);
}

void outlineTextAtCenter(int y, String txt) {
  int x = display.width() / 2 - txt.length() * 3;
  display.setTextColor(WHITE);
  textAt(x - 1, y, txt);
  textAt(x + 1, y, txt);
  textAt(x, y - 1, txt);
  textAt(x, y + 1, txt);
  display.setTextColor(BLACK);
  textAt(x, y, txt);
  display.setTextColor(WHITE);
}

void boldTextAtCenter(int y, String txt) {
  int x = display.width() / 2 - txt.length() * 3;
  textAt(x, y, txt);
  textAt(x + 1, y, txt);
}