/*
  Akana R1 Snake v1

  Snake Game for METE HOCA Akana R1

  Board : METE HOCA Akana R1 (https://www.metehoca.com/akana-r1-yeni-nesil-gelistirme-karti/)
  Dock  : -
  Core  : earlephilhower / arduino-pico (https://github.com/earlephilhower/arduino-pico)
  Required Libraries: Adafruit SSD1306 (https://github.com/adafruit/Adafruit_SSD1306)
                      Adafruit GFX (https://github.com/adafruit/Adafruit-GFX-Library)
  Settings: Select METE HOCA Akana R1 as the board.
            Set the I2C selector to the I2C0 position.

  Source: Enjoy-Mechatronics/Chaker Gamra / METE HOCA
  
  METE HOCA, 19 May 2024
  IG : https://www.instagram.com/metehoca/
  YT : https://www.youtube.com/MeteHoca
  WEB: https://www.metehoca.com/
*/

#include <Wire.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_GFX.h>

Adafruit_SSD1306 display(128, 64, &Wire, -1);

#define SNAKE_MAX_LENGTH 12
int snake_x[SNAKE_MAX_LENGTH];
int snake_y[SNAKE_MAX_LENGTH];
int snake_length = 2;
int score = 0;
int snake_dir = 0;  // 0 = right, 1 = up, 2 = left, 3 = down
int food_x;
int food_y;

void setup() {
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.display();

  pinMode(BTN_UP, INPUT);
  pinMode(BTN_DOWN, INPUT);
  pinMode(BTN_LEFT, INPUT);
  pinMode(BTN_RIGHT, INPUT);

  randomSeed(analogRead(0));
  food_x = random(0, display.width() / 8);
  food_y = random(0, display.height() / 8);

  snake_x[0] = 0;
  snake_y[0] = 0;
  snake_x[1] = 1;
  snake_y[1] = 0;

  attachInterrupt(digitalPinToInterrupt(BTN_UP), buttonUP, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_DOWN), buttonDN, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_LEFT), buttonLF, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_RIGHT), buttonRT, RISING);

  drawSnake();
  drawFood();
}

void buttonUP() { if (snake_dir != 3) snake_dir = 1; }
void buttonDN() { if (snake_dir != 1) snake_dir = 3; }
void buttonLF() { if (snake_dir != 0) snake_dir = 2; }
void buttonRT() { if (snake_dir != 2) snake_dir = 0; }

void loop() {
  delay(50);
  for (int i = snake_length - 1; i > 0; i--) {
    snake_x[i] = snake_x[i - 1];
    snake_y[i] = snake_y[i - 1];
  }

  switch (snake_dir) {
    case 0:
      snake_x[0]++;
      break;
    case 1:
      snake_y[0]--;
      break;
    case 2:
      snake_x[0]--;
      break;
    case 3:
      snake_y[0]++;
      break;
  }

  if (snake_x[0] < 0) {
    snake_x[0] = display.width() / 8 - 1;
  } else if (snake_x[0] >= display.width() / 8) {
    snake_x[0] = 0;
  }

  if (snake_y[0] < 0) {
    snake_y[0] = display.height() / 8 - 1;
  } else if (snake_y[0] >= display.height() / 8) {
    snake_y[0] = 0;
  }

  //Eat Apple
  if (snake_x[0] == food_x && snake_y[0] == food_y) {
    if (snake_length < SNAKE_MAX_LENGTH) {
      snake_length++;
    }
    score = score + 1;

    food_x = random(0, display.width() / 8);
    food_y = random(0, display.height() / 8);
  }

  for (int i = 1; i < snake_length; i++) {
    if (snake_x[0] == snake_x[i] && snake_y[0] == snake_y[i]) {
      gameOver();
      Reset();
    }
  }

  display.clearDisplay();
  drawSnake();
  drawFood();
  display.display();
}

void drawSnake() {

  for (int i = 0; i < snake_length; i++) {
    display.fillRect(snake_x[i] * 8, snake_y[i] * 8, 8, 8, WHITE);
  }
}

void drawFood() {
  display.fillRect(food_x * 8, food_y * 8, 8, 8, WHITE);
}

void gameOver() {
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(5, 15);
  display.println("Game Over!");
  display.setCursor(15, 35);
  display.print("Score: ");
  display.println(score);
  display.display();
  delay(3000);
}

void Reset() {
  score = 0;
  food_x = random(0, display.width() / 8);
  food_y = random(0, display.height() / 8);
  snake_length = 2;
  snake_dir = 0;  // 0 = right, 1 = up, 2 = left, 3 = down
  snake_x[0] = 0;
  snake_y[0] = 0;
  snake_x[1] = 1;
  snake_y[1] = 0;
}